function distsMin = ComputeMinDistFromWalls(WorldElementIndices)
% uses ScanPoints_global_Dump

global World
global ScanPoints_global_Dump

% ScanPoints_global  -- A 2xN matrix of global coordinates of scan points

% Structure World
%                .N                 -- Number of elements in the world
%                .Center(:,k)     -- A central point of the k-th element
%                .RMax(k)       -- An upperbound on the 'radius' of the k-th element
%                .element{k}        -- A structure defining a line/curve
%
%                           .type       =  'LineSegment'
%                           .ends       -- A 2x2 matrix each each column representing a vertex
%
%                           .type       = 'CircularArk'
%                           .c          -- center
%                           .r          -- radius
%                           .anglerange -- a 1x2 vector of [startAngle, endAngle]
%

% Structure ScanPoints_global_Dump
%                                 .count    -- size of the dump
%                                 .coords   -- coordinates of the points

pointCount = ScanPoints_global_Dump.count; %size(ScanPoints_global_Dump.coords, 2);
distsMin = inf(1, pointCount);

if nargin < 1
    WorldElementIndices = 1:World.N;
end

for k = WorldElementIndices
    
    %%ThisElementDists = ComputeDistsFromElement(ScanPoints_global_Dump, k);
    dists = distsMin;

    
    switch World.element{k}.type


        case 'LineSegment'

            LineEnds = World.element{k}.ends;
            SegmentVector = World.element{k}.etc.SegmentVector; %LineEnds(:,2) - LineEnds(:,1);
            SegmentLength = World.element{k}.etc.SegmentLength; %sqrt(sum(SegmentVector.^2));
            SegmentUnitVector = World.element{k}.etc.SegmentUnitVector; %SegmentVector / SegmentLength;
            pointsVector_1 = ScanPoints_global_Dump.coords - repmat(LineEnds(:,1), [1 pointCount]);
            pointsProjections = sum( pointsVector_1 .* repmat(SegmentUnitVector, [1 pointCount]) );

            outsideTags_1 = (pointsProjections < 0);
            outsideTags_2 = (pointsProjections > SegmentLength);
            insideTags = not( outsideTags_1 | outsideTags_2);

            dists = inf(1, pointCount);
            dists(outsideTags_1) = sqrt(sum( (ScanPoints_global_Dump.coords(:,outsideTags_1) - repmat(LineEnds(:,1), [1 sum(outsideTags_1)])) .^ 2));
            dists(outsideTags_2) = sqrt(sum( (ScanPoints_global_Dump.coords(:,outsideTags_2) - repmat(LineEnds(:,2), [1 sum(outsideTags_2)])) .^ 2));

            insideTagsCount = sum(insideTags);
            if insideTagsCount
%                 a = LineEnds(2,2) - LineEnds(2,1);
%                 b = LineEnds(1,1) - LineEnds(1,2);
%                 c = LineEnds(2,1)*LineEnds(1,2) - LineEnds(1,1)*LineEnds(2,2);
%                 mag = sqrt(a*a + b*b);
                an = World.element{k}.etc.an; %a / mag;
                bn = World.element{k}.etc.bn; %b / mag;
                cn = World.element{k}.etc.cn; %c / mag;
                dists(insideTags) = abs( an*ScanPoints_global_Dump.coords(1,insideTags) + bn*ScanPoints_global_Dump.coords(2,insideTags) + cn );
            end


        case 'CircularArk'

            ArkAngles = World.element{k}.anglerange;
            if ArkAngles(2) <= ArkAngles(1) % This should have been done before - no need to do it here again!
                ArkAngles(2) = ArkAngles(2) + 2*pi;
            end

            rs = sqrt(sum( (ScanPoints_global_Dump.coords - repmat(World.element{k}.c, [1 pointCount])) .^ 2));
            ths = atan2( ScanPoints_global_Dump.coords(2,:)-World.element{k}.c(2) , ScanPoints_global_Dump.coords(1,:)-World.element{k}.c(1) );
            lowAngleIndices = (ths <= ArkAngles(1));
            ths(lowAngleIndices) = ths(lowAngleIndices) + 2*pi;

            outsideTags = (ths > ArkAngles(2));
            insideTags = not(outsideTags);

            outsidePointsCount = sum(outsideTags);
            dists = inf(1, pointCount);
            endPoint1 = World.element{k}.etc.endPoints(:,1); %World.element{k}.c + World.element{k}.r*[cos(ArkAngles(1)); sin(ArkAngles(1))];
            endPoint2 = World.element{k}.etc.endPoints(:,2); %World.element{k}.c + World.element{k}.r*[cos(ArkAngles(2)); sin(ArkAngles(2))];
            dists(outsideTags) = sqrt(min([ sum( (ScanPoints_global_Dump.coords(:,outsideTags) - repmat(endPoint1, [1 outsidePointsCount])) .^ 2);
                sum( (ScanPoints_global_Dump.coords(:,outsideTags) - repmat(endPoint2, [1 outsidePointsCount])) .^ 2)  ]));
            dists(insideTags) = abs( rs(insideTags) - World.element{k}.r );

    end

    
    distsMin = min([dists;distsMin]);
    
end



% =================================================================================
% % 
% % function dists = ComputeDistsFromElement(points, k)
% % global World
% % 
% % pointCount = size(points, 2);
% % 
% % switch World.element{k}.type
% %     
% %     
% %     case 'LineSegment'
% %         
% %         LineEnds = World.element{k}.ends;
% %         SegmentVector = LineEnds(:,2) - LineEnds(:,1);
% %         SegmentLength = sqrt(sum(SegmentVector.^2));
% %         SegmentUnitVector = SegmentVector / SegmentLength;
% %         pointsVector_1 = points - repmat(LineEnds(:,1), [1 pointCount]);
% %         pointsProjections = sum( pointsVector_1 .* repmat(SegmentUnitVector, [1 pointCount]) );
% %         
% %         outsideTags_1 = (pointsProjections < 0);
% %         outsideTags_2 = (pointsProjections > SegmentLength);
% %         insideTags = not( outsideTags_1 | outsideTags_2);
% %         
% %         dists = inf(1, pointCount);
% %         dists(outsideTags_1) = sqrt(sum( (points(:,outsideTags_1) - repmat(LineEnds(:,1), [1 sum(outsideTags_1)])) .^ 2));
% %         dists(outsideTags_2) = sqrt(sum( (points(:,outsideTags_2) - repmat(LineEnds(:,2), [1 sum(outsideTags_2)])) .^ 2));
% %         
% %         insideTagsCount = sum(insideTags);
% %         if insideTagsCount
% %             a = LineEnds(2,2) - LineEnds(2,1);
% %             b = LineEnds(1,1) - LineEnds(1,2);
% %             c = LineEnds(2,1)*LineEnds(1,2) - LineEnds(1,1)*LineEnds(2,2);
% %             mag = sqrt(a*a + b*b);
% %             an = a / mag;
% %             bn = b / mag;
% %             cn = c / mag;
% %             dists(insideTags) = abs( an*points(1,insideTags) + bn*points(2,insideTags) + cn );
% %         end
% %         
% %         
% %     case 'CircularArk'
% %         
% %         ArkAngles = World.element{k}.anglerange;
% %         if ArkAngles(2) <= ArkAngles(1)
% %             ArkAngles(2) = ArkAngles(2) + 2*pi;
% %         end
% %         
% %         rs = sqrt(sum( (points - repmat(World.element{k}.c, [1 pointCount])) .^ 2));
% %         ths = atan2( points(2,:)-World.element{k}.c(2) , points(1,:)-World.element{k}.c(1) );
% %         lowAngleIndices = (ths <= ArkAngles(1));
% %         ths(lowAngleIndices) = ths(lowAngleIndices) + 2*pi;
% %         
% %         outsideTags = (ths > ArkAngles(2));
% %         insideTags = not(outsideTags);
% %         
% %         outsidePointsCount = sum(outsideTags);
% %         dists = inf(1, pointCount);
% %         endPoint1 = World.element{k}.c + World.element{k}.r*[cos(ArkAngles(1)); sin(ArkAngles(1))];
% %         endPoint2 = World.element{k}.c + World.element{k}.r*[cos(ArkAngles(2)); sin(ArkAngles(2))];
% %         dists(outsideTags) = sqrt(min([ sum( (points(:,outsideTags) - repmat(endPoint1, [1 outsidePointsCount])) .^ 2);
% %                                          sum( (points(:,outsideTags) - repmat(endPoint2, [1 outsidePointsCount])) .^ 2)  ]));
% %         dists(insideTags) = abs( rs(insideTags) - World.element{k}.r );
% %         
% % end
