function wt = Particles_ComputeWeights(ScanPoints_local)
global World Particles
global ScanPoints_global_Dump

% Structure Particles
%                    .N     -- number of particles
%                    .pos   -- 2xN vector : position of the particles
%                    .th    -- 1xN vector : theta-coordinate of the particles
%                    .wt    -- 1xN vector : weight associated with each particle
%                    .WorldBelonging  -- World.N x Particles.N matrix

% Structure ScanPoints_global_Dump
%                                 .count    -- size of the dump
%                                 .coords   -- coordinates of the points
%

% ScanPoints_local is a 2xM matrix with each column representing a scan point
% in the local coordinate frame of the particle
% The transformation from local to global coordinates is governed by the function
% ScanPoints_global = TransformScanPoints(ScanPoints_local, x, y, th)

wt = Particles.wt;
if isempty(ScanPoints_local)
    return;
end
    
for a = 1:Particles.N
    
    ScanPoints_global_Dump.coords = TransformScanPoints(ScanPoints_local, Particles.pos(1,a), ...
                                                    Particles.pos(2,a), Particles.th(a));
    ScanPoints_global_Dump.count = size(ScanPoints_global_Dump.coords, 2);
    dists_this = ComputeMinDistFromWalls(find(Particles.WorldBelonging(:,a))'); %^=^
    wt(a) = 1/mean(dists_this.^2); %^=^
    
end


% ====================================================
