
function RobotComputeAndSetVelocity_LaserAndParticleFilterLocalization(doParticleWeightUpdate, doParticleResampling, speed, InitialLocalizationOnly, ParticlesAfterDownsample)
global Control
global NowCoord GoalCoord NowTheta NoiseCovariences
global Particles

persistent callCount
persistent LastCommand LastLastCommand LastRead LastEstimate


if nargin < 1
    doParticleWeightUpdate = 0;
end
if nargin < 2
    doParticleResampling = 0;
end
if nargin < 3
    speed = Control.params.RobotSpeed;
elseif isempty(speed)
    speed = Control.params.RobotSpeed;
end
if nargin < 4
    InitialLocalizationOnly = 0;
elseif isempty(InitialLocalizationOnly)
    InitialLocalizationOnly = 0;
end
if nargin < 5
    ParticlesAfterDownsample = [];
end

if isempty(callCount)
    callCount = 0;
else
    callCount = callCount + 1;
end

if isempty(LastCommand)
    LastCommand.vel = [1,1] * 1e-10;
    LastCommand.timestamp = clock;
    LastLastCommand = LastCommand;
end
if isempty(LastEstimate)
    LastEstimate.timestamp = clock;
end


% robotData = Robot_Communicate('getData_Pose');
% Find robotData from laser scan and particle filter
% Initiate particles
InitParticleCount = 2000; %10000;
if callCount == 0
    if isempty(NowCoord)
        Particles = InitiateParticles(InitParticleCount);
    else
        if isempty(NowTheta)
            Noise3Sd = 2 * sqrt(NoiseCovariences); % * 3
            Noise3Sd(3) = min([Noise3Sd(3), pi]);
            poseNoise = [-Noise3Sd, Noise3Sd];
            Particles = InitiateParticles(InitParticleCount, poseNoise, NowCoord);
            %~Particles = InitiateParticles(10000, [-3 3; -3 3; -pi pi], NowCoord);
            %Particles = InitiateParticles(10000, [-6+NowCoord(1) 6+NowCoord(1); -6+NowCoord(2) 6+NowCoord(2); -pi pi]);
        else
            Noise3Sd = 2 * sqrt(NoiseCovariences); % * 3
            Noise3Sd(3) = min([Noise3Sd(3), pi/2]); % pi
            poseNoise = [-Noise3Sd, Noise3Sd];
            Particles = InitiateParticles(InitParticleCount, poseNoise, [NowCoord; NowTheta]);
            %~Particles = InitiateParticles(10000, [-3 3; -3 3; -pi/3 pi/3], [NowCoord; NowTheta]);
            %Particles = InitiateParticles(10000, [-6+NowCoord(1) 6+NowCoord(1); -6+NowCoord(2) 6+NowCoord(2); -pi+NowTheta pi+NowTheta]);
        end
    end
end

%CommandNoiseModel = sqrt([mean(NoiseCovariences(1:2)); NoiseCovariences(3)]); % [0.1; 0.1];
CommandNoiseModel = sqrt([5.0; 2.0]); % Noise model to be used!
Particles_Propagate(LastLastCommand.vel, etime(LastCommand.timestamp,LastEstimate.timestamp), CommandNoiseModel);
LastEstimate.timestamp = clock;
Particles_Propagate(LastCommand.vel, etime(LastEstimate.timestamp,LastCommand.timestamp), CommandNoiseModel);

if doParticleWeightUpdate || callCount == 0
    
    LastRead.timestamp = clock; % we assume this is almost same as current value of LastEstimate.timestamp
    laserData = Robot_Communicate('getData_LaserLocal');
%     if callCount
%         RobotComputeAndSetVelocity_LaserAndParticleFilterLocalization(0,0);
%     end
    
    Particles_ComputeWorldBelonging;
    NewWt = Particles_ComputeWeights(laserData.LaserPoints_local);
    %figure; plot(laserData.LaserPoints_local(1,:), laserData.LaserPoints_local(2,:), 'r.');
    Particles.wt = Particles.wt .* NewWt;
    Particles.wt = Particles.wt / max(Particles.wt);

end


RuntimeParticleCount = 100;
if doParticleResampling || callCount == 0
    
    if InitialLocalizationOnly
        if ~isempty(ParticlesAfterDownsample)
            % ParticlesAfterDownsample.SurviveRate
            % ParticlesAfterDownsample.FinalCount
            SurviveParticleCount = round(Particles.N / ParticlesAfterDownsample.SurviveRate);
            if ParticlesAfterDownsample.FinalCount > SurviveParticleCount
                OffspringPerSurvivor = round((ParticlesAfterDownsample.FinalCount / SurviveParticleCount) - 1);
                Particles_Resample(1/ParticlesAfterDownsample.SurviveRate, OffspringPerSurvivor);
            else
                Particles_Resample(1/round(Particles.N/ParticlesAfterDownsample.FinalCount), 0);
            end
        end
        % No resampling
    elseif Particles.N > RuntimeParticleCount+1 %0.9*InitParticleCount %callCount == 0
        SubsequentDownsampleFraction = round(Particles.N / RuntimeParticleCount);
        if isempty(NowCoord)
            Particles_Resample(1/SubsequentDownsampleFraction, 0);
        else
            Particles_Resample(1/SubsequentDownsampleFraction, 0);
        end
    else
        Particles_Resample(1/10, 9);
        %Particles_Resample(1/10, 9, [-0.2 0.2; -0.3 0.2; -pi/6 pi/6]);
    end
    
end


[MaxWt, BestParticleIndx] = max(Particles.wt);
robotData.p = Particles.pos(:,BestParticleIndx);
robotData.theta = Particles.th(BestParticleIndx);
NowCoord = robotData.p; % Test!
NowTheta = robotData.theta; % Test!

Particles.WinnerIndex = BestParticleIndx;
Particles.WinnerConfidence = Particles.N / sum(Particles.wt .* ( (Particles.pos(1,:)-robotData.p(1)).^2 + (Particles.pos(2,:)-robotData.p(2)).^2 ) ...
                                                        .* abs(Particles.th-robotData.theta) );
                                    
speed = speed * (1 + 3*min([1, max([0, Particles.WinnerConfidence-1000])/4000]));
                                                    

if ~InitialLocalizationOnly
    
    currentPos = robotData.p + Control.params.FeedbackLinearizationDistance*[cos(robotData.theta); sin(robotData.theta)];
    searchAheadPointIndices = ...
        Control.PathPoints.NowIndex:min([ Control.PathPoints.NowIndex + round(Control.params.SearchAheadDistForClosestPoint/Control.params.PathDiscretizationResolution), ...
        size(Control.PathPoints.points,2) ]);
    distSqFromAheadPoints = sum( (Control.PathPoints.points(:,searchAheadPointIndices) - repmat(currentPos, [1 length(searchAheadPointIndices)])) .^ 2);
    [nearestPtDistSq, nearestPtIndex] = min(distSqFromAheadPoints);

    Control.PathPoints.NowIndex = searchAheadPointIndices(nearestPtIndex);

    targetPointIndex = min([ Control.PathPoints.NowIndex+round(Control.params.TargetPointDistanceAhead/Control.params.PathDiscretizationResolution), ...
        size(Control.PathPoints.points,2) ]);
    targetPoint = Control.PathPoints.points(:,targetPointIndex);

    globalVel = targetPoint - currentPos;
    globalVel = speed * globalVel / norm(globalVel);

    localVel = [cos(robotData.theta), sin(robotData.theta); -sin(robotData.theta), cos(robotData.theta)] * globalVel;
    lastVel = [localVel(1), localVel(2)/Control.params.FeedbackLinearizationDistance];
    if callCount <= 5
        lastVel = lastVel / 4;
    end
    
else
    
    lastVel(1) = 1e-6;
    lastVel(2) = speed;
    
end

if abs(lastVel(2)) > Control.params.MaxTurnRate
    %'Turnrate scaled',
    lastVel = Control.params.MaxTurnRate * lastVel / abs(lastVel(2));
end

LastLastCommand = LastCommand;
LastCommand.vel = lastVel;
LastCommand.timestamp = clock;
Robot_Communicate('setVelocity', lastVel);
