function [NewWorld, ThisMergeWt] = MergeElementsToWorld(elements, OldWorld, wt, NowPos)

NewWorld = OldWorld;
MergeCost = 0;

for a = 1:length(elements)
    
    foundMatch = 0;
    
    for b = 1:NewWorld.N

        % Check distance from this element and continue if it is too far!
        if norm(elements{a}.Center - NewWorld.element{b}.Center) > (elements{a}.RMax + NewWorld.element{b}.RMax)
            continue;
        end
        
        er1 = sum(elements{a}.etc.SegmentUnitVector .* NewWorld.element{b}.etc.SegmentUnitVector); % cos of the angle
        %er2 = norm(elements{a}.Center - OldWorld.element{b}.Center);
        dists_ab = findDist_PointsElement(elements{a}, NewWorld.element{b}.ends);
        dists_ba = findDist_PointsElement(NewWorld.element{b}, elements{a}.ends);
        er2 = min([dists_ab, dists_ba]);

        MergeCost = MergeCost + ComputeMismatchCost(elements{a}, NewWorld.element{b}, NowPos, er1, er2);

        if ~foundMatch

            % Merge elements into one
            if abs(er1) > 0.95 && er2 < 0.01
                pts = [NewWorld.element{b}.ends, elements{a}.ends];
                centerPt = mean(pts')';
                dirn = mean([elements{a}.etc.SegmentUnitVector, sign(er1)*NewWorld.element{b}.etc.SegmentUnitVector]')';
                dists = sum( [dirn,dirn,dirn,dirn] .* (pts - [centerPt,centerPt,centerPt,centerPt]) );
                [d1,i1] = min(dists);
                [d2,i2] = max(dists);
                p1 = pts(:,i1);
                p2 = pts(:,i2);
                element_new = GenerateWorldElement([p1,p2], 'LineSegment');
                NewWorld.element{b} = element_new;
                NewWorld.wt(b) = min([NewWorld.wt(b),wt]);
                foundMatch = 1;
                %break;
                
            % Recreate element - create 2 elements
            elseif abs(er1) > 0.85 && er2 < 0.01 % $$$
                [far_d1, far_i1] = max(dists_ab); % NewWorld.element{b} -- 1 ; elements{a} -- 2
                [far_d2, far_i2] = max(dists_ba);
                [near_d1, near_i1] = min(dists_ab);
                [near_d2, near_i2] = min(dists_ba);
                p1 = NewWorld.element{b}.ends(:,far_i1);
                pmid = mean([NewWorld.element{b}.ends(:,near_i1), elements{a}.ends(:,near_i2)]')';
                p2 = elements{a}.ends(:,far_i2);
                element_new1 = GenerateWorldElement([p1,pmid], 'LineSegment');
                element_new2 = GenerateWorldElement([pmid,p2], 'LineSegment');
                NewWorld.element{b} = element_new1;
                NewWorld.wt(b) = min([NewWorld.wt(b),wt]);
                NewWorld.N = NewWorld.N + 1;
                NewWorld.element{NewWorld.N} = element_new2;
                NewWorld.wt(NewWorld.N) = NewWorld.wt(b);
                foundMatch = 1;
                %break;
                
            end

        end

    end

    if ~foundMatch
        NewWorld.N = NewWorld.N + 1;
        NewWorld.element{NewWorld.N} = elements{a};
        NewWorld.wt(NewWorld.N) = wt;
    end
    
end

ThisMergeWt = exp(-MergeCost*100); % 1,


% ===============================================================================

function cost = ComputeMismatchCost(e1, e2, p, cosAng, separation)
% e1 cannot be 'behind' e2, i.e. if e1 and e2 are not intersecting,
% thr triangle formed by lines joining p and ends of e1 and e1 itself
% cannot contain any part of e2
cost = 0;

IntersectThresh = 0.02;
SeparationThresh = 0.01; % $$$
ParallelThresh = 0.85;

if (cosAng < ParallelThresh) || (separation > SeparationThresh)
    % Do this only if e1 and e2 are distinct elements

    s1.v = e1.ends;
    s1.a = e1.etc.an;
    s1.b = e1.etc.bn;
    s1.c = e1.etc.cn;
    s2.v = e2.ends;
    s2.a = e2.etc.an;
    s2.b = e2.etc.bn;
    s2.c = e2.etc.cn;

    [result, intersectPoint] = isIntersect_Segments_2(s1, s2);

    if result
        % segments intersect

        dist_intersect = sqrt(sum(([s1.v, s2.v] - [intersectPoint,intersectPoint,intersectPoint,intersectPoint]) .^2));
        sort_dist_intersect = sort(dist_intersect);

        cost = cost + (max([sort_dist_intersect(2)-IntersectThresh, 0])^2) * (1-abs(cosAng));

    end
    
        % Segments are different - check for obstruction

%         sgnP = sign(s1.a*p(1) + s1.b*p(2) + s1.c);
%         sgn2a = sign(s1.a*s2.v(1,1) + s1.b*s2.v(2,1) + s1.c);
%         sgn2a = sign(s1.a*s2.v(1,2) + s1.b*s2.v(2,2) + s1.c);

        e1_p_1 = GenerateWorldElement([p,s1.v(:,1)], 'LineSegment');
        e1_p_2 = GenerateWorldElement([p,s1.v(:,2)], 'LineSegment');
        
        triangleS(1) = s1;
        triangleS(2).v = [p,s1.v(:,1)];
        triangleS(2).a = e1_p_1.etc.an;
        triangleS(2).b = e1_p_1.etc.bn;
        triangleS(2).c = e1_p_1.etc.cn;
        triangleS(3).v = [p,s1.v(:,2)];
        triangleS(3).a = e1_p_2.etc.an;
        triangleS(3).b = e1_p_2.etc.bn;
        triangleS(3).c = e1_p_2.etc.cn;
        InPoint = mean([p, s1.v]')';
        for ss = 1:3
            inSign(ss) = triangleS(ss).a*InPoint(1) + triangleS(ss).b*InPoint(2) + triangleS(ss).c;
            p1Sign(ss) = triangleS(ss).a*s2.v(1,1) + triangleS(ss).b*s2.v(2,1) + triangleS(ss).c;
            p2Sign(ss) = triangleS(ss).a*s2.v(1,2) + triangleS(ss).b*s2.v(2,2) + triangleS(ss).c;
        end
        p1_inside = (sign(inSign) == sign(p1Sign));
        p2_inside = (sign(inSign) == sign(p2Sign));
        lenCost = 0;
        if all(p1_inside) && all(p2_inside) % if e2 entirely lies inside e1
            lenCost = lenCost + norm(s2.v(:,1)-s2.v(:,2));
        elseif all(p1_inside) % if an vertex of e2 lies inside e1
            lenCost = lenCost + min(abs(p1Sign(p1_inside)));
        elseif all(p2_inside)
            lenCost = lenCost + min(abs(p2Sign(p2_inside)));
        else % if e2 'cuts' the triangle!
            [resultA, intersectPointA] = isIntersect_Segments_2(triangleS(2), s2);
            [resultB, intersectPointB] = isIntersect_Segments_2(triangleS(3), s2);
            [resultC, intersectPointC] = isIntersect_Segments_2(s1, s2);
            if resultA && resultB
                lenCost = lenCost + norm(intersectPointA-intersectPointB);
            elseif resultB && resultC
                lenCost = lenCost + norm(intersectPointC-intersectPointB);
            elseif resultA && resultC
                lenCost = lenCost + norm(intersectPointC-intersectPointA);
            end
        end
        
        cost = cost + max([lenCost-IntersectThresh, 0])^2;
        cost = sqrt(cost);
        
    %end
    
end
