function VGraph_Initiate
%{
******************************************************************************************
*                                                                                        *
*    A* search with homotopy class constraints                                           *
*    Copyright (C) 2010  Subhrajit Bhattacharya                                          *
*                                                                                        *
*    This program is free software: you can redistribute it and/or modify                *
*    it under the terms of the GNU General Public License as published by                *
*    the Free Software Foundation, either version 3 of the License, or                   *
*    (at your option) any later version.                                                 *
*                                                                                        *
*    This program is distributed in the hope that it will be useful,                     *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of                      *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                       *
*    GNU General Public License for more details <http://www.gnu.org/licenses/>.         *
*                                                                                        *
*                                                                                        *
*    Contact: subhrajit@gmail.com, http://fling.seas.upenn.edu/~subhrabh/                *
*                                                                                        *
*                                                                                        *
******************************************************************************************
%}


global VGraph
global NowCoord GoalCoord

% Structure VGraph
%                 .ActiveIndices        -- A vector containing the indices of the active clusters ( = [1,2,ObstaclesInSight+2])
%                 .VertexCluster{i}     -- All the vertices associated with cluster i
%                                           ( i=1 -> start position - the only dynamic cluster
%                                             i=2 -> goal position
%                                             i=p -> Obstacle number p-2 )
%                                  .Vertices -- 3xV matrix. First 2 elements of each column represent coordinates of the vertices
%                                                           Last element indicating the validity of the vertices
%
%                 .InterClusterEdges{i,j}  -- Edges joining clusters i and j [IMPORTANT: i <= j ]
%                                        .isComputed -- Indicates whether these edges have been computed before (Check using isfield)
%                                        .Edges -- A 6xE matrix of edges along with cost
%                                                                   = [ v_i1    v_i2    ...   ;
%                                                                       v_j1    v_j2    ...   ;
%                                                                       cost_1  cost_2  ...   ;
%                                                                       a_1     a_2     ...   ;
%                                                                       b_1     b_2     ...   ;
%                                                                       c_1     c_2     ...   ; ]
%                                                                   v_kp is the index of the vertex in VGraph.VertexCluster(k).Vertices
%
%                 .LeastCostPath(k) -- contains information about the k-th segment of the path
%                                  .i    -- Index of cluster from where the edge originates
%                                  .v_i  -- Vertex index of cluster i
%                                  .j    -- Index of cluster where the edge ends
%                                  .v_j  -- Vertex index of cluster j
%                                  .e    -- Edge index in VGraph.InterClusterEdges{i,j}.Edges
%                                  .direction -- 1 or -1
%
% c_k = norm( VGraph.VertexCluster{i}.Vertices(1:2,v_ik) - VGraph.VertexCluster{j}.Vertices(1:2,v_jk) )
%

VGraph.ActiveIndices = [1, 2];

VGraph.VertexCluster{1}.Vertices = [NowCoord; 1];
VGraph.VertexCluster{2}.Vertices = [GoalCoord; 1];

VGraph.InterClusterEdges{1,1}.isComputed = 0;
VGraph.InterClusterEdges{1,1}.Edges = [];

VGraph.InterClusterEdges{2,2}.isComputed = 1;
VGraph.InterClusterEdges{2,2}.Edges = [];

VGraph.InterClusterEdges{1,2}.isComputed = 0;
VGraph.InterClusterEdges{1,2}.Edges = [1; 1; ...
                        ComputeAuxiliaryEdgeColumn(VGraph.VertexCluster{1}.Vertices(1:2,1), VGraph.VertexCluster{2}.Vertices(1:2,1))];
