
% ======================================================================
% ======================================================================
function [PathNodes, LVal] = search
% start and goal: [cluster_index, vertex_index]
start = [1 1];
stop = [2 1];

global z0 z0countA z0countB
global VGraph N M BlockedLVals TargetLVal
global ExistingLVals
global g_score h_score f_score i_score came_from
global openset closedset

N = length(VGraph.VertexCluster);
M = 0;
for a = 1:N
    M = max([M, size(VGraph.VertexCluster{a}.Vertices, 2)]);
end

V = 1000;
z0countA = floor(length(z0)/2);
z0countB = length(z0) - z0countA - 1;

closedset = [];                 % The set of nodes already evaluated.
openset = X2I(start,0); % The set of tentative nodes to be evaluated.

buffSize = 1000;
g_score = inf(1,buffSize);
f_score = inf(1,buffSize);
came_from = zeros(1,buffSize);
%i_score = zeros(1,buffSize) + zeros(1,buffSize)*i;

g_score(X2I(start,0)) = 0;                        % Distance from start along optimal path.
h_score(X2I(start,0)) = heuristic_estimate_of_distance(start, stop);
f_score(X2I(start,0)) = h_score(X2I(start,0));           % Estimated total distance from start to goal through y.
%i_score(X2I(start,0)) = 0+0i;

while length(openset)>0

    xindTmp = find(f_score(openset)==min(f_score(openset)));
    xind = openset(xindTmp(1));

    if (~isempty(TargetLVal) && xind==X2I(stop,TargetLVal))
        StopIndex = xind;
        ExpansionCount = length(closedset);
        break; %return;
    elseif (isempty(TargetLVal) && all(stop==I2X(xind)))
        [x,xL] = I2X(xind);
        % adaptiveThresh = thresh * 10^log(max([1 abs(Lval(a))]));
        if ~any(abs(ExistingLVals{x(1),x(2)}(xL) - BlockedLVals) < 0.1)
            StopIndex = xind;
            ExpansionCount = length(closedset);
%             x,
%             xL,
%             xind,
%             ExistingLVals{x(1),x(2)},
             %pause;
            break; %return;
        end
    end

    openset = setdiff(openset,xind);
    closedset(end+1) = xind;
    for yind = neighbor_nodes(xind)
        if yind > buffSize
            buffSize = yind + 1000;
            came_from(end+1:buffSize) = 0;
            g_score(end+1:buffSize) = inf;
            f_score(end+1:buffSize) = inf;
        end

        if any(yind==closedset)
            continue;
        end

        x = I2X(xind);
        y = I2X(yind);
        %tentative_i_score = i_score(xind) + Delta_i(x,y);
        tentative_g_score = g_score(xind) + dist_between(x,y); % + V * abs(i_score(yind) + Delta_i(y,x) + Stop2StartI);

        if ~any(yind==openset)
            openset(end+1) = yind;
            tentative_is_better = true;
        elseif tentative_g_score < g_score(yind)
            tentative_is_better = true;
        else
            tentative_is_better = false;
        end

        if tentative_is_better==true
            came_from(yind) = xind;
            g_score(yind) = tentative_g_score;
            %old_i_score = i_score(yind);
            %i_score(yind) = tentative_i_score; % + Delta_i(x,y);
            h_score(yind) = heuristic_estimate_of_distance(y, stop);
            f_score(yind) = g_score(yind) + h_score(yind);
        end

    end
end

%StopIndex = X2I(start,0);
%ExpansionCount = length(closedset);

PathNodes = reconstruct_path(StopIndex, start);
PathNodes = PathNodes(end:-1:1,:);

LVal = CplxI(PathNodes);



function zI = CplxI(PathNodes)
global VGraph
zI = 0+0*i;
for a = 1:(size(PathNodes,1)-1)
    zI = zI + Delta_i(PathNodes(a,:),PathNodes(a+1,:));
    %disp(VGraph.VertexCluster{PathNodes(a,1)}.Vertices(1,PathNodes(a,2)) + i*VGraph.VertexCluster{PathNodes(a,1)}.Vertices(2,PathNodes(a,2)));
    %disp(VGraph.VertexCluster{PathNodes(a+1,1)}.Vertices(1,PathNodes(a+1,2)) + i*VGraph.VertexCluster{PathNodes(a+1,1)}.Vertices(2,PathNodes(a+1,2)));
end

%[xx,LNo,LVal] = I2X(PathNodes(end,:)),

%PathNodes,
%LVal,
%pause;

% ======================================================================

function ii = X2I(xx, Lval)
global N M
global ExistingLVals
thresh = 0.1; %0.1;

if isempty(ExistingLVals)
    ExistingLVals = cell(N,M);
end

if isempty(Lval) && size(xx,1)==1
    layerNo = 1;
    layerNos = 1:length(ExistingLVals{xx(1,1),xx(1,2)});
    ii = N*M*(layerNos-1) + N*(xx(1,2)-1) + xx(1,1);
else
    for a=1:size(xx,1)
        adaptiveThresh = thresh; % * (max([1 abs(Lval(a))]));
        ExistFlags = (abs(ExistingLVals{xx(a,1),xx(a,2)} - Lval(a)) < adaptiveThresh);
        if any(ExistFlags)
            layerNo(a) = find(ExistFlags);
            ExistingLVals{xx(a,1),xx(a,2)}(layerNo(a)) = 0.7*ExistingLVals{xx(a,1),xx(a,2)}(layerNo(a)) + 0.3*Lval(a);
        else
            layerNo(a) = length(ExistingLVals{xx(a,1),xx(a,2)}) + 1;
            ExistingLVals{xx(a,1),xx(a,2)}(layerNo(a)) = Lval(a);
        end
    end

    ii = N*M*(layerNo-1) + N*(xx(:,2)-1) + xx(:,1);

end


% ======================================================================

function [xx,LNo,LVal] = I2X(ind)
global N M ExistingLVals
ii = mod(ind-1,N*M) + 1;
xx(:,1) = mod(ii-1, N) + 1;
xx(:,2) = floor((ii-1)/N) + 1;
if nargout>1
    LNo = floor((ind-1)/(N*M)) + 1;
end
if nargout>2
    for a=1:length(ind)
        LVal(a) = ExistingLVals{xx(a,1),xx(a,2)}(LNo(a));
    end
end

% ======================================================================

function h = heuristic_estimate_of_distance(a, b)
% [cluster_index, vertex_index]
global VGraph
dx = VGraph.VertexCluster{a(1)}.Vertices(1,a(2)) - VGraph.VertexCluster{b(1)}.Vertices(1,b(2));
dy = VGraph.VertexCluster{a(1)}.Vertices(2,a(2)) - VGraph.VertexCluster{b(1)}.Vertices(2,b(2));
h = sqrt(dx*dx + dy*dy);

% ======================================================================

function d = dist_between(a, b)
global VGraph
dx = VGraph.VertexCluster{a(1)}.Vertices(1,a(2)) - VGraph.VertexCluster{b(1)}.Vertices(1,b(2));
dy = VGraph.VertexCluster{a(1)}.Vertices(2,a(2)) - VGraph.VertexCluster{b(1)}.Vertices(2,b(2));
d = sqrt(dx*dx + dy*dy);

% ======================================================================

function yi = neighbor_nodes(xi)
global VGraph N M
global ExistingLVals
global ComputedTransitions
% 8-connected grid
[x, xL] = I2X(xi); % [cluster_index, vertex_index]
yi = [];
neiCount = 0;

% [a,b]
for a = 1:N % different clusters
    if a<=x(1)
        ii=a; jj=x(1);
        searchIndx = 2;
        fetchIndex = 1;
    else
        ii=x(1); jj=a;
        searchIndx = 1;
        fetchIndex = 2;
    end
    
    if isempty(VGraph.InterClusterEdges{ii,jj}.Edges)
        continue;
    end
    
    for b = VGraph.InterClusterEdges{ii,jj}.Edges(fetchIndex,(VGraph.InterClusterEdges{ii,jj}.Edges(searchIndx,:)==x(2))) % vertex
        neiCount = neiCount + 1;

        xt(1) = a;
        xt(2) = b;
        if xt(1)>N || xt(1)<1 || xt(2)>M || xt(2)<1
            continue;
        end

        si = SpaceInd(x);
        sit = SpaceInd(xt);
        %{
        if si > sit
            arrowSgn = -1;
            tmpsi=si; si=sit; sit=tmpsi;
        else
            arrowSgn = 1;
        end
        if length(ComputedTransitions)>=si && length(ComputedTransitions{si})>=neiCount && ComputedTransitions{si}(neiCount)~=0
            dI = arrowSgn * ComputedTransitions{si}(neiCount);
        else
            dI = Delta_i(x, xt);
            ComputedTransitions{si}(neiCount) = arrowSgn * dI;
        end
        %}
        if size(ComputedTransitions,1)>=si && size(ComputedTransitions,2)>=sit && ~isempty(ComputedTransitions{si, sit})
            dI = ComputedTransitions{si,sit};
        else
            dI = Delta_i(x, xt);
            ComputedTransitions{si,sit} = dI;
            ComputedTransitions{sit,si} = -dI;
        end

        tryLval = ExistingLVals{x(1),x(2)}(xL) + dI;
        yi(end+1) = X2I(xt,tryLval);
    end
end



function ind = SpaceInd(x)
global N M
ind = (x(2)-1)*N + x(1);



% ======================================================================

function dI = Delta_i(x1, x2)
global VGraph

z1 = VGraph.VertexCluster{x1(1)}.Vertices(1,x1(2)) + i*VGraph.VertexCluster{x1(1)}.Vertices(2,x1(2));
z2 = VGraph.VertexCluster{x2(1)}.Vertices(1,x2(2)) + i*VGraph.VertexCluster{x2(1)}.Vertices(2,x2(2));

% Adaptive step size
%zm = (z1+z2)/2;
%Ofzm = log(abs(f(zm)));
%del = 0.1*0.1^ceil(min([max([1 Ofzm-6]) 2])); %pause;
del = 1e-3; %0.001;
% Integration
Dz = z2 - z1;
if abs(Dz) > 0
    del = del / abs(Dz);
end
delDz = del*Dz;
dI = 0+0*i;
p = (del/2):del:(1-del/2);
    cz = z1 + p*Dz;
    dI = sum(f(cz))*delDz;
%end

function fz = f(z)
global z0 z0countA z0countB
fz = ones(1, length(z));
for a=1:length(z0)
    fz = fz .* (z - z0(a));
end
%fz = ((((z-(20+i*20)).*z).^(length(z0)/2))) ./ fz;
fz = ( ((z-(20+i*20)).^z0countA) .* (z.^z0countB) ) ./ fz;
%fz = (sqrt(M*N)^(length(z0)/2)) ./ fz;

function zI = CplxI(PathNodes)
zI = 0+0*i;
for a = 1:(size(PathNodes,1)-1)
    zI = zI + Delta_i(PathNodes(a,:),PathNodes(a+1,:));
end

% ======================================================================


function p = reconstruct_path(stop, start, TargetLVal)
global came_from

if length(stop) > 1
%     stop = StopIndex;
%     if nargin < 3
%         TargetLVal = [];
%     end
%     currind = X2I(stop,TargetLVal);
%     p = stop;
elseif length(stop)==1
    currind = stop;
    p = I2X(currind);
end

startind = X2I(start,0);

while came_from(currind)>0 %&& currind~=startin
    p(end+1,:) = I2X(came_from(currind));
    currind = came_from(currind);
end
