function GenerateWorld(indices)
% Generates the structure World (boundaries of obstacles) from UObstacles (uninflated obstacles)
global UObstacles World

% UObstacles -- a structure defining various shapes
%           .N       -- number of obstacles
%           .uobs{a} -- cell array of obstacles
%
%                   .CreatedFromPoints -- boolian describing whether it is created from scan points
%                   .points -- (optional - required only if .CreatedFromPoints is 1) the coordinates of points
%                   .Geometry
%
%                            .type = 'circle'
%                            .r -- radius of circle
%                            .c -- center of circle
%
%                            .type = 'straight_rectangle'
%                            .xrange -- a vector [xMin, xMax]
%                            .yrange -- a vector [yMin, yMax]
%
%                            .type = 'polygon'
%                            .vertices -- 2xN matrix of coordinates

% Structure World
%                .N                 -- Number of elements in the world
%                .element{k}        -- A structure defining a line/curve
%
%                           .type       =  'LineSegment'
%                           .ends       -- A 2x2 matrix each each column representing a vertex
%
%                           .type       = 'CircularArk'
%                           .c          -- center
%                           .r          -- radius
%                           .anglerange -- a 1x2 vector of [startAngle, endAngle]

if nargin < 1
    indices = 1:UObstacles.N;
end
if ~isfield(World, 'N')
    World.N = 0;
end


for a = indices
    
    switch UObstacles.uobs{a}.Geometry.type
        
        case 'circle'
            
            World.N = World.N + 1;
            World.element{World.N}.type = 'CircularArk';
            World.element{World.N}.r = UObstacles.uobs{a}.Geometry.r;
            World.element{World.N}.c = UObstacles.uobs{a}.Geometry.c;
            World.element{World.N}.anglerange = [-pi, pi];
            
        case 'straight_rectangle'
            xrange = UObstacles.uobs{a}.Geometry.xrange;
            yrange = UObstacles.uobs{a}.Geometry.yrange;
            World.N = World.N + 1;
            World.element{World.N}.type = 'LineSegment';
            World.element{World.N}.ends = [xrange(1) xrange(1); yrange(1) yrange(2)];
            World.N = World.N + 1;
            World.element{World.N}.type = 'LineSegment';
            World.element{World.N}.ends = [xrange(1) xrange(2); yrange(2) yrange(2)];
            World.N = World.N + 1;
            World.element{World.N}.type = 'LineSegment';
            World.element{World.N}.ends = [xrange(2) xrange(2); yrange(2) yrange(1)];
            World.N = World.N + 1;
            World.element{World.N}.type = 'LineSegment';
            World.element{World.N}.ends = [xrange(2) xrange(1); yrange(1) yrange(1)];
            
    end
    
end
