function PredictedParticles = PredictParticles(StartParticles, vel, dt)

% The StartParticles are propagated with velocity 'vel' for time 'dt' to obtain new particle positions

% Structure Particles
%                    .N     -- number of particles
%                    .pos   -- 2xN vector : position of the particles
%                    .th    -- 1xN vector : theta-coordinate of the particles
%                    .wt    -- 1xN vector : weight associated with each particle

PredictedParticles.th = StartParticles.th + vel.omega*dt;

shiftVector = (vel.v/vel.omega) * [ sin(PredictedParticles.th)-sin(StartParticles.th) ; ...
                                    -cos(PredictedParticles.th)+cos(StartParticles.th) ];

PredictedParticles.pos = StartParticles.pos + shiftVector;

PredictedParticles.N = StartParticles.N;
PredictedParticles.wt = StartParticles.wt;
