function result = isPointInsideObstacle(points)
global Obstacles

% Structure Obstacles
%                    .N                -- number of obstacles
%                    .obs{i}           -- cell array of structure that defines i-th obstacle
%                            .shape    -- Can be 'circle' or 'polygon' (present code supports only polygons)
%                       polygon:
%                            .Vertices -- 3xV matrix. First 2 elements of each column represent coordinates of the vertices
%                                                           Last element indicating the validity of the vertices
%

% points is a 2xN matrix of point coordinates
% returns a 1xN vector of booleans

pointsCount = size(points,2);
result = false(1, pointsCount);

for a = 1:Obstacles.N
    
    v1 = repmat(Obstacles.obs{a}.Vertices(1:2,1), [1 pointsCount]) - points;
    vA = v1;
    vCount = 0;
    OrientSigns = [];
    for b = [2:size(Obstacles.obs{a}.Vertices,2), 1]
        vB = repmat(Obstacles.obs{a}.Vertices(1:2,b), [1 pointsCount]) - points;
        vCount = vCount + 1;
        OrientSigns(vCount,:) = sign( vA(1,:).*vB(2,:) - vA(2,:).*vB(1,:) );
        vA = vB;
    end
    thisObstacleTags = (abs(sum(OrientSigns))==vCount);
    result = result | thisObstacleTags;
    
end
