function GenerateObstacles(indices)
% Generates the structure Obstacles (inflated obstacles) from UObstacles (uninflated obstacles)
global Obstacles UObstacles
global Boundary
global Control

% Structure Control
%                  .params
%                         .RobotRadius
%                         .FeedbackLinearizationDistance
%                         .Clearance_KnownObstacles
%                         .Clearance_GeneratedObstacles
%                         .PathDiscretizationResolution


if nargin < 1
    indices = 1:UObstacles.N;
end
if ~isfield(Obstacles,'N')
    Obstacles.N = max(indices);
else
    Obstacles.N = max([Obstacles.N, indices]);
end

for a = indices
    
    if UObstacles.uobs{a}.CreatedFromPoints
        inc = Control.params.RobotRadius + Control.params.FeedbackLinearizationDistance + Control.params.Clearance_GeneratedObstacles;
    else
        inc = Control.params.RobotRadius + Control.params.FeedbackLinearizationDistance + Control.params.Clearance_KnownObstacles;
    end
    
    switch UObstacles.uobs{a}.Geometry.type
        
        case 'circle'
            poly_n = 8;
            angles = (0:(poly_n-1))*2*pi/poly_n;
            polyRad = (UObstacles.uobs{a}.Geometry.r + inc)/cos(pi/poly_n);
            points = [ UObstacles.uobs{a}.Geometry.c(1)+polyRad*cos(angles) ; UObstacles.uobs{a}.Geometry.c(2)+polyRad*sin(angles) ];
            
        case 'straight_rectangle'
            xrange = UObstacles.uobs{a}.Geometry.xrange;
            yrange = UObstacles.uobs{a}.Geometry.yrange;
            points = [ xrange(1)-inc , xrange(2)+inc , xrange(2)+inc , xrange(1)-inc ;
                       yrange(1)-inc , yrange(1)-inc , yrange(2)+inc , yrange(2)+inc ];
                   
    end
    
    % Assign validity tags : for now check only whether inside workspace boundary
    validity = (Boundary(1,1) < points(1,:)) & (points(1,:) < Boundary(1,2)) & ...
                    (Boundary(2,1) < points(2,:)) & (points(2,:) < Boundary(2,2));
    %validity = ones([1 size(points,2)]);
                
    if any(validity)
        %Obstacles.N = Obstacles.N + 1;
        Obstacles.obs{a}.shape = 'polygon';
        Obstacles.obs{a}.Vertices = [ points ; validity ];
        VGraph.VertexCluster{a+2}.Vertices = Obstacles.obs{a}.Vertices;
    end
    
end

Obstacles.N = length(Obstacles.obs);

