function InitiateUObstacle(spec_filename)
global UObstacles
global NowCoord GoalCoord Boundary

% UObstacles -- a structure defining various shapes
%           .N       -- number of obstacles
%           .uobs{a} -- cell array of obstacles
%
%                   .CreatedFromPoints -- boolian describing whether it is created from points
%                   .points -- (optional - required only if .CreatedFromPoints is 1) the coordinates of points
%                   .Geometry
%
%                            .type = 'circle'
%                            .r -- radius of circle
%                            .c -- center of circle
%
%                            .type = 'straight_rectangle'
%                            .xrange -- a vector [xMin, xMax]
%                            .yrange -- a vector [yMin, yMax]
%
%                            .type = 'polygon'
%                            .vertices -- 2xN matrix of coordinates
%

% Load data from specification file to create the UObstacles

% Loading specification file
try
    fid = fopen(spec_filename);
    C = textscan(fid, '%s %f %f %f %f');
    fclose(fid);
catch
    disp('Error: Requested specification file not found! Please check filename and path.');
    disp('The specification file should be under the current directory!');
    return;
end

UObstacles.N = 0;
for a = 1:length(C{1})
    
    thisName = C{1}(a);
    switch thisName{1}
        
        case 'boundary'
            Boundary(1,1) = C{2}(a); % xMin
            Boundary(2,1) = C{3}(a); % yMin
            Boundary(1,2) = C{4}(a); % xMax
            Boundary(2,2) = C{5}(a); % yMax
            
        case 'start'
            NowCoord(1,1) = C{2}(a);
            NowCoord(2,1) = C{3}(a);
            
        case 'end'
            GoalCoord(1,1) = C{2}(a);
            GoalCoord(2,1) = C{3}(a);
            
        otherwise
            % Obstacle
            if strcmp(thisName{1}(1:8),'obstacle')
                UObstacles.N = UObstacles.N + 1;
                UObstacles.uobs{UObstacles.N}.CreatedFromPoints = 0;
                UObstacles.uobs{UObstacles.N}.points = [];
                if isempty(C{5}(a)) || isnan(C{5}(a))
                    UObstacles.uobs{UObstacles.N}.Geometry.type = 'circle';
                    UObstacles.uobs{UObstacles.N}.Geometry.c(1,1) = C{2}(a);
                    UObstacles.uobs{UObstacles.N}.Geometry.c(2,1) = C{3}(a);
                    UObstacles.uobs{UObstacles.N}.Geometry.r = C{4}(a);
                else
                    UObstacles.uobs{UObstacles.N}.Geometry.type = 'straight_rectangle';
                    UObstacles.uobs{UObstacles.N}.Geometry.xrange = [ C{2}(a)-C{4}(a)/2 , C{2}(a)+C{4}(a)/2 ];
                    UObstacles.uobs{UObstacles.N}.Geometry.yrange = [ C{3}(a)-C{5}(a)/2 , C{3}(a)+C{5}(a)/2 ];
                end
            end
            
    end
end
