function ret = DrawScene(action, params)
global UObstacles UBoundary Obstacles VGraph World Particles
global NowCoord GoalCoord Boundary
persistent fig_H UObstaclesGeometry_H UObstaclesPoints_H Obstacles_H VGraph_H Path_H World_H Particles_H

switch action
    
    
    
    case 'initiate'
        fig_H = figure;
        hold on;
        axis equal;
        axis([UBoundary(1,1), UBoundary(1,2), UBoundary(2,1), UBoundary(2,2)]);
        
        
    case 'NowPoint'
        figure(fig_H);
        hold on;
        plot(NowCoord(1),NowCoord(2),'b.');

    case 'GoalPoint'
        figure(fig_H);
        hold on;
        plot(GoalCoord(1),GoalCoord(2),'b.');
        
        
    case 'UObstacles_Geometry'
        figure(fig_H);
        hold on;
        if nargin > 1
            uobsToDraw = params;
        else
            uobsToDraw = 1:UObstacles.N;
        end
        for a = uobsToDraw
            if a <= length(UObstaclesGeometry_H)
                try
                    delete(UObstaclesGeometry_H{a});
                end
            end
            switch UObstacles.uobs{a}.Geometry.type
                case 'circle'
                    pts = Circle(UObstacles.uobs{a}.Geometry.r, UObstacles.uobs{a}.Geometry.c);
                    UObstaclesGeometry_H{a} = plot(pts(1,:),pts(2,:),'b-');
                case 'straight_rectangle'
                    UObstaclesGeometry_H{a} = plot(UObstacles.uobs{a}.Geometry.xrange([1 2 2 1 1]), UObstacles.uobs{a}.Geometry.yrange([1 1 2 2 1]), 'b-');
            end
        end
        

    case 'UObstacles_Points'
        figure(fig_H);
        hold on;
        if nargin > 1
            uobsToDraw = params;
        else
            uobsToDraw = 1:UObstacles.N;
        end
        for a = uobsToDraw
            if UObstacles.uobs{a}.CreatedFromPoints
                if a <= length(UObstaclesPoints_H)
                    delete(UObstaclesPoints_H{a});
                end
                UObstaclesPoints_H{a} = plot(UObstacles.uobs{a}.points(1,:),UObstacles.uobs{a}.points(2,:),'c.');
            end
        end
        
        
    case 'Obstacles'
        figure(fig_H);
        hold on;
        if nargin > 1
            obsToDraw = params;
        else
            obsToDraw = 1:Obstacles.N;
        end
        for a = obsToDraw
            try
                delete(Obstacles_H{a});
            end
            if any( a+2 == VGraph.ActiveIndices )
                lineWidth = 0.5;
            else
                lineWidth = 0.5;
            end
            Obstacles_H{a} = plot([Obstacles.obs{a}.Vertices(1,:), Obstacles.obs{a}.Vertices(1,1)], ...
                            [Obstacles.obs{a}.Vertices(2,:), Obstacles.obs{a}.Vertices(2,1)], 'r-', 'LineWidth', lineWidth);
        end
        
        
    case 'VGraph'
        figure(fig_H);
        hold on;
        VGraph.ActiveIndices = sort(VGraph.ActiveIndices);
        for i = 1:size(VGraph.InterClusterEdges,1)
            for j = i:size(VGraph.InterClusterEdges,2)
                % i = VGraph.ActiveIndices(a);
                % j = VGraph.ActiveIndices(b);
                if i <= size(VGraph_H,1) && j <= size(VGraph_H,2)
                    for c = 1:length(VGraph_H{i,j})
                        try
                            delete(VGraph_H{i,j}(c));
                        end
                    end
                end
                try
                    for c = 1:size(VGraph.InterClusterEdges{i,j}.Edges, 2)
                        vi = VGraph.VertexCluster{i}.Vertices(1:2,VGraph.InterClusterEdges{i,j}.Edges(1,c));
                        vj = VGraph.VertexCluster{j}.Vertices(1:2,VGraph.InterClusterEdges{i,j}.Edges(2,c));
                        VGraph_H{i,j}(c) = plot([vi(1),vj(1)], [vi(2),vj(2)], 'r-');
                    end
                end
            end
        end
        
        
    case 'Path'
        figure(fig_H);
        hold on;
        for a = 1:length(Path_H)
            try
                delete(Path_H{a});
            end
        end
        for a = 1:length(VGraph.LeastCostPath)
            Path_H{a} = plot([ VGraph.VertexCluster{VGraph.LeastCostPath(a).i}.Vertices(1,VGraph.LeastCostPath(a).v_i), ...
                                VGraph.VertexCluster{VGraph.LeastCostPath(a).j}.Vertices(1,VGraph.LeastCostPath(a).v_j) ], ...
                                [ VGraph.VertexCluster{VGraph.LeastCostPath(a).i}.Vertices(2,VGraph.LeastCostPath(a).v_i), ...
                                VGraph.VertexCluster{VGraph.LeastCostPath(a).j}.Vertices(2,VGraph.LeastCostPath(a).v_j) ], ...
                                'r-', 'LineWidth', 4);
        end
        

    case 'World'
        figure(fig_H);
        hold on;
        for k = 1:World.N
            try
                delete(World_H{k});
            end
            switch World.element{k}.type
                case 'LineSegment'
                    World_H{k} = plot(World.element{k}.ends(1,:), World.element{k}.ends(2,:), 'b-');
                case 'CircularArk'
                    pts = Circle(World.element{k}.r, World.element{k}.c, World.element{k}.anglerange);
                    World_H{k} = plot(pts(1,:), pts(2,:), 'b-');
            end
        end


    case 'Particles'
        figure(fig_H);
        hold on;
        %[MaxWt, MaxIndx] = max(Particles.wt);
        MaxIndx = Particles.WinnerIndex;
        MaxWt = Particles.wt(MaxIndx);
%         for a=1:Particles.N
%             try
%                 delete(Particles_H.points{a});
%             end
%             Particles_H.points{a} = plot(Particles.pos(1,a), Particles.pos(2,a), '.', 'Color', (Particles.wt(a)/MaxWt)*[0 1 0]');
%         end
        try
            delete(Particles_H.points);
            delete(Particles_H.MostProbablePoint.point);
            delete(Particles_H.MostProbablePoint.text);
        end
        % The following takes substantial amount of time!
        plotIndx = (Particles.wt >= 0*MaxWt/100); %100
        plotCount = sum(plotIndx);
        Particles_H.points = plot([-1e6*ones(1,plotCount); Particles.pos(1,plotIndx)], [zeros(1,plotCount); Particles.pos(2,plotIndx)], '.');
        vv = 1 - (Particles.wt(plotIndx)/MaxWt).^(1/3);
        for a=1:plotCount
            ptcolor{a} = [vv(a); 1; vv(a)];
        end
        set(Particles_H.points, {'Color'}, ptcolor');
        Particles_H.MostProbablePoint.point = plot(Particles.pos(1,MaxIndx), Particles.pos(2,MaxIndx), 'b*');
        Particles_H.MostProbablePoint.text = text(Particles.pos(1,MaxIndx), Particles.pos(2,MaxIndx), ...
                                                        sprintf('[%0.3f]',Particles.WinnerConfidence), 'FontWeight','bold');
        ret = Particles.pos(:,MaxIndx);

    case 'Particles_Weights'
        figure(fig_H);
        hold on;
        maxWt = max(Particles.wt);
        for a=1:Particles.N
            if Particles.wt(a) < 0.8*maxWt
                continue;
            end
            try
                delete(Particles_H.wts{a});
            end
            Particles_H.wts{a} = text(Particles.pos(1,a), Particles.pos(2,a), sprintf('(%0.3f,%0.3f)',Particles.th(a),Particles.wt(a)));
        end

end
