function Particles_Resample(RetainFraction, OffspringRate, ParticleBoundary)
global Particles

% For maintaining same number of particles use RetainFraction = 1 / (OffspringRate+1)
% OffspringRate is an integer

% WARNING: wt and WorldBelonging information are destroyed!

% Structure Particles
%                    .N     -- number of particles
%                    .pos   -- 2xN vector : position of the particles
%                    .th    -- 1xN vector : theta-coordinate of the particles
%                    .wt    -- 1xN vector : weight associated with each particle
%                    .WorldBelonging  -- World.N x Particles.N matrix

if nargin < 2
    OffspringRate = round(1/RetainFraction) - 1;
end
if nargin < 3
    ParticleBoundary = [0 0; 0 0; 0 0]; %[-1 1; -1 1; -pi/2 pi/2];
end

MaxWt = max(Particles.wt);
[sortWts, sortIndx] = sort(Particles.wt, 'descend');
RetainCount = round(RetainFraction*Particles.N);
RetainIndex = sortIndx(1:RetainCount);

Particles.N = RetainCount;
Particles.pos = Particles.pos(:,RetainIndex);
Particles.th = Particles.th(:,RetainIndex);
Particles.wt = Particles.wt(:,RetainIndex);
Particles.WorldBelonging = Particles.WorldBelonging(:,RetainIndex);

if OffspringRate > 0
    for a = 1:RetainCount

        if norm(ParticleBoundary) ~= 0
            PossibleLocation = [Particles.pos(:,a); Particles.th(a)];
            NewParticles = InitiateParticles(OffspringRate, ParticleBoundary, PossibleLocation);
        else
            NewParticles.N = OffspringRate;
            NewParticles.pos = repmat(Particles.pos(:,a), [1 NewParticles.N]);
            NewParticles.th = repmat(Particles.th(a), [1 NewParticles.N]);
        end

        NewParticles.wt = repmat(Particles.wt(a), [1 NewParticles.N]);
        NewParticles.WorldBelonging = repmat(Particles.WorldBelonging(:,a), [1 NewParticles.N]);

        Particles.N = Particles.N + NewParticles.N;
        Particles.pos = [Particles.pos, NewParticles.pos];
        Particles.th = [Particles.th, NewParticles.th];
        Particles.wt = [Particles.wt, NewParticles.wt];
        Particles.WorldBelonging = [Particles.WorldBelonging, NewParticles.WorldBelonging];

    end
end
