function [PoseEstimate_new, PoseFilter] = KalmanFilter_pose(action, PoseFilter, PoseReading_new)

% Structure PoseReading_new
%                          .x
%                          .y
%                          .theta
%                          .TimeStamp

% Structure PoseFilter

%                     .PoseEstimate_Count
%                     .PoseEstimate_TimeStamps(k)                -- the time stamp of the k-th stored estimate
%                     .PoseEstimate(k)              -- Array that stores the recent 'PoseEstimate_Count' number of pose estimates
%                                     .x
%                                     .y
%                                     .theta

%                     .PoseReadings_Count
%                     .PoseReadings_TimeStamps(k)                -- the time stamp of the k-th reading
%                     .PoseReadings(k)              -- Array that stores the recent 'PoseReadings_Count' number of PoseReadings
%                                     .x
%                                     .y
%                                     .theta

%                     .Commands_Count
%                     .Commands_TimeStamps(k)                -- the time stamp of the k-th command issue
%                     .Commands(k)                  -- Array that stores the recent 'Commands_Count' number of Commands
%                                 .v
%                                 .omega


switch action
    
    case 'initiate'
        
        PoseFilter.PoseEstimate_Count = 3;
        PoseFilter.PoseEstimate_TimeStamps = -inf(1,PoseFilter.PoseEstimate_Count);
        PoseFilter.PoseReadings_Count = 3;
        PoseFilter.PoseReadings_TimeStamps = -inf(1,PoseFilter.PoseReadings_Count);
        PoseFilter.Commands_Count = 3;
        PoseFilter.Commands_TimeStamps = -inf(1,PoseFilter.Commands_Count);
        
    case 'update'
        
        % Include the latest reading
        [Sorted_PoseReadings_TimeStamp, Sorted_PoseReadings_TimeStamp_i] = sort(PoseFilter.PoseReadings_TimeStamps);
        replace_index = Sorted_PoseReadings_TimeStamp_i(1);
        PoseFilter.PoseReadings(replace_index).x = PoseReading_new.x;
        PoseFilter.PoseReadings(replace_index).y = PoseReading_new.y;
        PoseFilter.PoseReadings(replace_index).theta = PoseReading_new.theta;
        PoseFilter.PoseReadings_TimeStamps(replace_index) = PoseReading_new.TimeStamp;
        [Sorted_PoseReadings_TimeStamp, Sorted_PoseReadings_TimeStamp_i] = sort(PoseFilter.PoseReadings_TimeStamps);
        
        [Sorted_PoseEstimate_TimeStamp, Sorted_PoseEstimate_TimeStamp_i] = sort(PoseFilter.PoseEstimate_TimeStamps);
        if Sorted_PoseEstimate_TimeStamp(end) ~= Sorted_PoseReadings_TimeStamp(end-1)
            disp('WARNING: Inconsistency in time stamps!');
        end
        t1 = Sorted_PoseEstimate_TimeStamp(end);
        est1 = PoseFilter.PoseEstimate(Sorted_PoseEstimate_TimeStamp_i(end));
        t2 = Sorted_PoseReadings_TimeStamp(end);
        
        [Sorted_Commands_TimeStamp, Sorted_Commands_TimeStamp_i] = sort(PoseFilter.Commands_TimeStamps);
        Before_Commands_TimeStamps_i = Sorted_Commands_TimeStamp_i( (Sorted_Commands_TimeStamp <= t1) );
        Mid_Commands_TimeStamps_i = Sorted_Commands_TimeStamp_i( (t1 < Sorted_Commands_TimeStamp) & ...
                                                                       (Sorted_Commands_TimeStamp < t2) );
        PoseEstimate_new = est1;
        Command_this = PoseFilter.Commands(Before_Commands_TimeStamps_i(end));
        Command_this_t = PoseFilter.Commands_TimeStamps(Before_Commands_TimeStamps_i(end));
        t_last = t1;
        for i = Mid_Commands_TimeStamps_i
            start_t = max([Command_this_t, t1]);
            Command_next = PoseFilter.Commands(i);
            Command_next_t = PoseFilter.Commands_TimeStamps(i);
            end_t = min([Command_next_t, t2]);
            
            PoseEstimate_new = PoseEstimate_new + (end_t-start_t)*Command_this; % 
            
            Command_this = Command_next;
            Command_this_t = Command_next_t;
        
        
        replace_index = Sorted_PoseEstimate_TimeStamp_i(1);
        