function Particles_ComputeWorldBelonging
global Particles World

% Structure Particles
%                    .N     -- number of particles
%                    .pos   -- 2xN vector : position of the particles
%                    .th    -- 1xN vector : theta-coordinate of the particles
%                    .wt    -- 1xN vector : weight associated with each particle
%                    .WorldBelonging  -- World.N x Particles.N matrix

% Structure World
%                .N                 -- Number of elements in the world
%                .Center(:,k)     -- A central point of the k-th element
%                .RMax(k)       -- An upperbound on the 'radius' of the k-th element
%                .element{k}        -- A structure defining a line/curve
%
%                           .type       =  'LineSegment'
%                           .ends       -- A 2x2 matrix each each column representing a vertex
%
%                           .type       = 'CircularArk'
%                           .c          -- center
%                           .r          -- radius
%                           .anglerange -- a 1x2 vector of [startAngle, endAngle]

LaseMaxRange = 4.1;

ParticleXMat = repmat(Particles.pos(1,:), [World.N 1]);
ParticleYMat = repmat(Particles.pos(2,:), [World.N 1]);
WorldXMat = repmat(World.Center(1,:)', [1 Particles.N]);
WorldYMat = repmat(World.Center(2,:)', [1 Particles.N]);

DistSqMat = (ParticleXMat-WorldXMat).^2 + (ParticleYMat-WorldYMat).^2;
DistSqThreshMat = repmat(((World.RMax + LaseMaxRange).^2)', [1 Particles.N]);

Particles.WorldBelonging = (DistSqMat <= DistSqThreshMat);
